pro crossnoise,degree,printer,wild,fmin,fmax
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; NAME:
;	CROSSNOISE
; CALLING SEQUENCE:
;	crossnoise,degree,printer,wild,fmin,fmax
; PURPOSE:
;	Plot cross spectra as a function of frequency
; INPUTS:
;	degree		degree of the mode
;	printer		printer name
;	wild		wild card of the filename ('specsoho'+wild)
;	fmin		frequency min for plotting (in microHz)
;	famx		frequency max for plotting (in microHz)
; OPTIONAL KEYWORDS:
;	None
; OUTPUTS:
;	None
; OPTIONAL OUTPUT:
;	None
; LIMITATIONS:
;	Size of the array (?)
; COMMONS:
;	None
; MODIFICATION HISTORY:
;	Beta 1: WRITTEN, Thierry Appourchaux, June,  1, 1997
;------------------------------------------------------------------------------
	nn=N_params()
	if (nn LT 5) then begin
		print,''
		if (nn EQ 0) then begin
			read,'give the degree of the mode= ',degree
			print,''
			degree=floor(degree)
			nn=nn+1
		endif
		if (nn EQ 1) then begin
			printer=''
		        read,'give the printer output as a string (press return to redirect to your monitor)= ',printer
			print,''
			nn=nn+1
		endif		
		if (nn EQ 2) then begin
			wild=''
		  	read,'give the wid card of your file as a string= ',wild
			print,''
			nn=nn+1
	        endif
		if (nn EQ 3) then begin
		  	read,'fmin= ',fmin
			print,''
			nn=nn+1
	        endif
		if (nn EQ 4) then begin
		  	read,'fmax= ',fmax
			print,''
			nn=nn+1
	        endif

		
	endif

	if (degree gt 3) then begin
		Nplot=3
	endif else begin
		Nplot=2
	endelse

	splotstring=['r','i','t']

	splot=0.
	while ((splot ne 1.) and (splot ne 2.) and (splot ne 3.)) do begin
	Read,'Which one to plot: Real (1), Imaginary (2) or Real+Imaginary (3)=',splot
	print,splot
	endwhile

	Read,'Between real-real(0) or real-imag(1)',srealimag

	if (srealimag eq 0) then begin
		sri=[0,0]
	endif else begin
		sri=[0,1]
	endelse


	z=findfile('specsoho'+wild+strtrim(degree,1))

	if (z(0) ne '') then begin
		a=readfits('specsoho'+wild+strtrim(degree,1))
	endif else begin
		z=pickfile()
		a=readfits(z)
	endelse

; ************************	
	Print,'The need for sign correction can be checked by looking at the Real (1) and the Imaginary (2) before looking at the sum (3), check it out!  This one here is specific to the LOI!'
	Read,'Sign correction (1 if Yes)=',scor
	if (scor eq 1) then begin
		print,'Sign correction being done...'
		a(*,0:degree-1,1)=-temporary(a(*,0:degree-1,1))
	endif

	Read,'Clean data from m (or l) leaks? (1 if Yes)=',clean
	
; ************************

	b=a

	Npoints=N_elements(a)/(2*degree+1)/2

	zz=fltarr(Npoints,2*degree+1)
	zn=fltarr(Npoints,2*degree+1)

	resol=1000000./Npoints/2./60.


		

; *******************************************************************


; Leakage matrices for LOI

if (degree eq 1) then begin

		ctnm1=  [[1.0000000,0.0000000,0.475],$
			[0.0000000,1.0000000,0.0000000],$     			         			  
			[0.475,0.0000000,1.000000]]

endif




if (degree eq 2) then begin

		ctnm1=     [[1.0000000,0.0000000,-0.30791966,0.0000000,-0.21587904],$
       			    [0.0000000,1.0000000,0.0000000,0.57568890,0.0000000],$
     			    [-0.30791966,0.0000000,1.000000,0.0000000,-0.30791966],$
       			    [0.0000000,0.57568890,0.0000000,1.0000000,0.0000000],$
     			    [-0.21587904,0.0000000,-0.30791966,0.0000000,1.000000]]

endif


if (degree eq 3) then begin



		ctnm1=     [[0.99999999,0.0000000,-0.19631255,0.0000000,0.12230649,0.0000000,-0.0044199663],$
       			    [0.0000000,0.99999996,0.0000000,0.46915087,0.0000000,-0.19569330,0.0000000],$
     			    [-0.19631255,0.0000000,1.0000000,0.0000000,0.54446038,0.0000000,0.12230649],$
       			    [0.0000000,0.46915087,0.0000000,0.99999999,0.0000000,0.46915087,0.0000000],$
		            [0.12230649,0.0000000,0.54446038,0.0000000,1.0000000,0.0000000,-0.19631255],$
       			    [0.0000000,-0.19569330,0.0000000,0.46915087,0.0000000,0.99999996,0.0000000],$
   			    [-0.0044199663,0.0000000,0.12230649,0.0000000,-0.19631255,0.000000,0.99999999]]

endif

		


; for GONG data	


;		print,ctnm1

	i=long(0)

; Clean data by applying the inverse of the leakage matrix

	if (clean eq 1) then begin
		read,'Use LOI leakage matrix? (1 if yes)',leak_loi
		If (leak_loi ne 1) then begin
			Print,'Leakage matrix computation valid for GONG only!'
			ct,degree,65.2,53.5,ctnm1
		endif

		if (degree eq 1) then begin
		read,'Redefine l=1 leakage matrix? (1 is yes)',read_alpha
		if (read_alpha eq 1) then begin
			ctnm1=fltarr(3,3)
			for i=0,2 do begin
				ctnm1(i,i)=1.0
			endfor
			Print,'Now we redefine the leakage matrix for l=1'
			read,'Read Alpha (leakage between m=-1 and m=+1)=',alpha
			ctnm1(0,2)=alpha
			ctnm1(2,0)=alpha
		endif
		endif
		print,ctnm1
		ctnm1inv=invert(ctnm1)
		print,'Start cleaning'
		for i=long(0),long(Npoints-1) do begin
			for k=0,1 do begin
				b(i,*,k)= ctnm1inv ## reform(a(i,*,k))
			endfor
		endfor
		print,'End cleaning'
		writefits,'specsoho'+wild+'new'+strtrim(degree,1),b
		a=b
	endif
		

; *******************************************************************************

	print,'Resolution=',resol

	x=findgen(Npoints)*resol

; **** Here we construct the cross spectra

	!p.multi=[0,Nplot,Nplot]

	for m=-degree,0 do begin
		!p.multi=[0,Nplot,Nplot]
		if (printer ne '') then begin
			set_plot,'ps',/interpolate
			file='noise'+wild+strtrim(abs(m),1)+'.ps'
			print,'filename= ',file
			device,/landscape,filename=file
		endif	
		for mp=-degree,degree do begin


		if (splot eq 1) then begin
			aa=smooth(a(*,m+degree,sri(0))*a(*,mp+degree,sri(1)),10)
			bb=smooth(a(*,m+degree,0)^2,10)
		endif
		if (splot eq 2) then begin
			aa=smooth(a(*,m+degree,1-sri(0))*a(*,mp+degree,1-sri(1)),10)
			bb=smooth(a(*,m+degree,1)^2,10)
		endif
		if (splot eq 3) then begin
			aa=smooth(a(*,m+degree,sri(0))*a(*,mp+degree,sri(1))+a(*,m+degree,1-sri(0))*a(*,mp+degree,1-sri(1)),10)
			bb=smooth(a(*,m+degree,0)^2+a(*,m+degree,1)^2,10)
		endif

		if (bb(0) eq 0.) then begin
			bb(0)=bb(1)
		endif

		zn(*,mp+degree)=smooth(aa/bb,32)

		index=where((x gt fmin) and (x lt fmax))

		print,moment(zn(index,mp+degree))




		plot,x,zn(*,mp+degree),xrange=[fmin,fmax],title='Cross spectrum between '+strtrim(m,1)+' and '+strtrim(mp,1),yrange=[-1.0,1.2],ystyle=1,xtitle='Frequency (in !4l!6Hz)',xstyle=1

		
		

		endfor
		if (printer ne '') then begin
			device,/close
			print,'Now printing...'
			spawn,/sh, 'lp -c -d '+printer+' '+file
			set_plot,'x
		endif

		read,blurp



	endfor	


end









pro echelleplotpile,a,degree,orderbegin,orderend,windowi,echelletitle
;
; frequency of the n=21
;
	if (windowi gt 135.) then begin
		windowi=135.
	endif

	delta=[134.8,134.8,135.1,135.4,135.5,136.2,136.7,137.2,137.7,138.2,138.7]

	del=delta(degree)
	
	window=windowi/2.

	freq=[3033,3098,3160,3218,3271,3322,3371,3417,3462,3509,3549]
	Na=N_elements(a)/(2*degree+1)
	resol=1000000./60./Na/2.
	print,'resolution',resol
	Nz=floor(del/resol)+1
	b=fltarr(Nz,11*(2.*degree+1))
	middlefreq=del/2./resol

	freqstart=(freq(degree)-5.*del-del/2.)/resol
	for im=0,2.*degree do begin

		for i=0,10 do begin
			b(*,i+im*11)=a(freqstart+i*Nz:freqstart+(i+1)*Nz-1,im)
		endfor
	endfor
	zz=congrid(b,Nz,33*(2.*degree+1))

	uu=zz(middlefreq-window/resol:middlefreq+window/resol,*)

	if (degree eq 1) then begin
	ytick=[' ','-1',' ','0',' ','1',' ']
tvframemod,uu,xrange=[-window,window],yrange=[-degree-0.5,degree+0.5],title=echelletitle,charsize=1.0,ytitle='Azimuthal order',xtitle='Frequency (in !4l!6Hz)',/bar,ytickname=ytick
	endif else begin
	tvframemod,uu,xrange=[-window,window],yrange=[-degree-0.5,degree+0.5],title=echelletitle,charsize=1.0,ytitle='Azimuthal order',xtitle='Frequency (in !4l!6Hz)',/bar

	endelse

end

