;********************************************************************
;
; Routine for fitting the SOHO LOI spectra
;
; degree is the degree (sic)
;
; wild is a string for various file name (i.e. 'new', 'old', etc...) 
;
; istart is the frequency number where should start the iteration
;
; tol is the tolerance for the Power algorithm
;
;*********************************************************************

pro gongamp,degree,wild,istart,tol,printer,window,asym
        common crosstalk,ctnm1
        common matbruit,croiseen
	common speed,Neval
        common nb_bruit, nbbruit
	common profile,profilename
	common lmin,Fmin

	nn=N_params()
	if (nn LT 9) then begin
		print,''
		if (nn EQ 0) then begin
			read,'give the degree of the mode= ',degree
			print,''
			degree=floor(degree)
			nn=nn+1
		endif
		if (nn EQ 1) then begin
			wild=''
		  	read,'give the wid card of your file as a string= ',wild
			print,''
			nn=nn+1
		endif
		if (nn EQ 2) then begin
		     	read,'give the starting point for the fit= ',istart
			print,''
			istart=floor(istart)
			nn=nn+1
		endif
		if (nn EQ 3) then begin
		        read,'give tolerance for the Powell algorithm (1d-08 works well on a Sparc20...)= ',tol
			print,''
			nn=nn+1
		endif
		if (nn EQ 4) then begin
			printer=''
		        read,'give the printer output as a string (press return to redirect to your monitor)= ',printer
			print,''
			nn=nn+1
		endif
		if (nn EQ 5) then begin
		        read,'window size (in microHz) ',window
			print,''
			nn=nn+1
		endif
		if (nn EQ 6) then begin
		        read,'Number of independent noise (3 is the usual for the LOI)',nbbruit
			print,''
			nn=nn+1
		endif
		if (nn EQ 7) then begin
		        read,'Asymmetry (1 if yes)',asym
			print,''
		endif

        endif
	Neval=0.

; Set output file name

if (asym eq 1) then begin
	wild_ass='ass'
endif else begin
	wild_ass='noass'
endelse


;************ Frequency list *************

Central=[[0,1,2,3,4,5,6,7,8,0,10,11,12,13,14,15,16,17,18],$
		 [1613.46,1749.26,1885.06,2020.86,2156.67, 2291.95, 2425.68,2559.14, 2693.32,2828.13, 2963.26, 3098.16, 3233.21, 3368.64, 3504.39, 3640.27,3776.5,3913.53,4052],$
		 [1674.51,1810.27,1945.76,2082.08,2217.6,2352,2486,2619,2754,2889,3024,3159,3295,3430,3567,3703,3838,3976,4113],$
		 [1729.06,1865.28,2001.19,2137.75,2273.34,2408,2542,2676,2811,2947,3083,3218,3354,3490,3626,3761,3898,4035,4172],$
		 [1729.,1865.,2001.,2188,2324,2458,2593,2729,2864,3000,3136,3272,3408,3544,3682,3819,3958,4095,4232],$
		 [1729.,1865.,2001.,2235,2371,2506,2642,2777,2913,3050,3186,3323,3458,3594,3735,3872,4010,4151,4292],$
		 [1729.,1865.,2001.,2280,2416,2551,2687,2823,2960.69,3097.28,3234.04,3371,3508,3646,3784,3922,4061,4201,4342],$
		 [1729.,1865.,2001.,2322,2458,2594,2731,2868,3006,3143,3280,3417,3555,3693,3831,3970,4108,4249,4390]]


;***************


;**************** PLEASE ADAPT TO YOUR OWN ****************************
;*******************READING DATA FILE *********************************
;*********************DEFINE THE DATA AND THE XDATA *******************

wild1=wild

wild='/home/thierrya/SOHOdata/specsoho'+wild 
		filename=wild+strtrim(degree,1)

		z=findfile(filename)
		
		if (z(0) eq '') then begin
			filename=dialog_pickfile(filter='specsoho*')
		endif

		buffer=readfits(filename,zozo)
print, zozo
		Nfourier=N_elements(buffer)/(2*degree+1)/2
		resol=1000000./60./Nfourier/2.
		Npoints=floor(window/resol)
		Npoints=Npoints+ Npoints mod 2
		data=dcomplex(buffer(*,*,0),buffer(*,*,1))	
		xdata=(dindgen(Npoints)-Npoints/2)*resol


;**********************************************************************

;**** Leakage matrix for an 'ideal' case *****************


ctnm1=fltarr(2*degree+1,2*degree+1)

for i=0,2*degree do begin
	ctnm1(i,i)=1.0
endfor


;**** Noise covaraince matrix for the same 'ideal' case **********



if (nbbruit eq 2) then begin
croiseen=dblarr(2,2*degree+1,2*degree+1)
if (degree eq 1) then begin
	croiseen(0,*,*)=[[1.0000000,0.0000000,0.3],$
       			 [0.0000000,0.0000000,0.0000000],$
			 [0.3,0.0000000,1.0000000]]

	croiseen(1,*,*)=[[0.0000000,0.0000000,0.0],$
       			 [0.0000000,1.0000000,0.0000000],$
			 [0.0,0.0000000,0.0000000]]
     		

endif
endif

if (nbbruit eq 3) then begin
croiseen=dblarr(3,2*degree+1,2*degree+1)


if (degree eq 2) then begin
	croiseen(0,*,*)=[[1.0000000,0.0000000,0.2,0.0000000,0.3],$
       			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [0.3,0.0000000,0.2,0.0000000,1.0000000]]

	croiseen(1,*,*)=[[0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
       			 [0.0000000,1.0000000,0.0000000,-0.45000000,0.0000000],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [0.0000000,-0.45000000,0.0000000,1.0000000,0.0000000],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000]]
     		
	croiseen(2,*,*)=[[0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
       			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [0.5000000,0.0000000,1.0000000,0.0000000,0.5000000],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000]]


endif


endif

if (nbbruit eq 4) then begin
croiseen=dblarr(4,2*degree+1,2*degree+1)


if (degree eq 3) then begin
	croiseen(0,*,*)=[[1.0000000,0.0000000,0.3,0.0000000,-0.2,0.0000000,-0.1],$
       			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [-0.1,0.0000000,-0.2,0.0000000,0.3,0.0000000,1.0000000]]

	croiseen(1,*,*)=[[0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
       			 [0.0000000,1.0000000,0.0000000,0.5,0.0000000,0.3,0.0000000],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [0.0000000,0.3,0.0000000,0.5000000,0.0000000,1.0000000,0.0000000],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000]]
     		
	croiseen(2,*,*)=[[0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
       			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [0.34,0.0000000,1.0000000,0.0000000,-0.54,0.0000000,-0.30],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [-0.30,0.0000000,-0.54,0.0000000,1.0000000,0.0000000,0.34],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000]]

	croiseen(3,*,*)=[[0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
       			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [0.0000000,-0.59,0.0000000,1.0000000,0.0000000,-0.59,0.0000000],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000],$
			 [0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,0.0000000]]


endif


endif

print,ctnm1

read,blurp

print,croiseen

read,blurp



;-------------------------------- number of parameters
; 1 freq + 1 largeur + 5 a_i + 2*degree+1 amplis + 6 bruits+ 1 ass!


Nparam=2*degree+15

param=dblarr(Nparam)
error=dblarr(Nparam)


;********************* Here we set the fixed string 

 
string='00'			;  freq, width


maxp= 2*degree < 5		;  Here we define how many a_i are to be fitted

string_a=''

for i=1,maxp do begin
	string_a=string_a+'0'
endfor


for i=maxp+1,5 do begin
	string_a=string_a+'1'
endfor


string_ampl=''			;  Here we define how many amplitudes are to be fitted


for m=1,2*degree+1 do begin
	string_ampl=string_ampl+'0'
endfor


string_noise=''			;  Here we define how many noise are to be fitted

for i=1,nbbruit do begin
	string_noise=string_noise+'0'
endfor

print,string_noise,nbbruit


for i=nbbruit+1,6 do begin		 
	string_noise=string_noise+'1'
endfor

print,string_noise,nbbruit


;********************* Here we init the parameters

param(0)=0.					;Freq.

param(1)=alog(0.9)				;width

param(2)=0.410					;a_1

param(3:6)=0.					;a_i i>0

param(7:2*degree+7)=-1.0			;Amplitudes



param(2*degree+8:2*degree+13)=-100.		;noise

param(2*degree+8:2*degree+8+nbbruit-1)=-1.2	;noise


; Set output filename


filename='Gong_'+wild1+wild_ass+'_'+strtrim(degree,1)

afile=findfile(filename+'*')


print,afile

longueur=N_elements(param)+2

pparamout=dblarr(longueur,31)
eerrorout=dblarr(longueur,30)
paramout=dblarr(longueur,31)
errorout=dblarr(longueur,30)
        
start=0

if (strlen(afile(0)) ne 0) then begin
	pparamin=readfits(afile(0))
	eerrorin=readfits(afile(1))

	start=N_elements(pparamin)/(N_elements(param)+1)

	pparamout(*,0:start-1)=pparamin
	eerrorout(*,0:start-1)=eerrorin

print, '$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$'
;print, hdr

endif









for ifreq=istart,18 do begin

start=start+1



; assymmetry

	param(Nparam-1)=0.

	if (asym eq 1) then begin
		str_as='0'
	endif else begin
		str_as='1'
	endelse


am=0.
for i=0,2*degree do begin
      am=am+exp(param(7+i))/(2.*degree+1.)
endfor
param(7:2*degree+7)=alog(am)

br=0.
for i=0,nbbruit-1 do begin
      br=br+exp(param(2*degree+8+i))/(nbbruit*1.)
endfor
param(2*degree+8:2*degree+8+nbbruit-1)=alog(br)






Cen=Central(ifreq,degree)

Nstart=floor(Cen/resol)
Freqstart=Nstart*resol

print,"*******************************************"
print,""
print,"Central frequency ",Freqstart
print,""
print,"*******************************************"

print,tol

window=Npoints*resol
print,"Window size=",window


ydata=dcomplexarr(Npoints,2*degree+1)
ydata(0:Npoints-1,*)=data(Nstart-Npoints/2:Nstart+Npoints/2-1,*)

string_b='01111'




startstring=string+string_b+string_ampl+string_noise+str_as

print,startstring

print,string_noise

mm=max(abs(ydata))

ydata=ydata/mm

profilename='lorentzian_ass'

print,param

fitplot_gong,xdata,ydata,'crosstalkgong',param,error,degree,tol,Freqstart,'LOI '+wild,printer,fixed=startstring,nodfp='10',/as

lmin1=Fmin



profilename='lorentzian_ass'

startstring=string+string_a+string_ampl+string_noise+str_as

fitplot_gong,xdata,ydata,'crosstalkgong',param,error,degree,tol,Freqstart,'LOI '+wild,printer,fixed=startstring,nodfp='10',/as

lmin2=Fmin

Print,'Delta lmin=',(lmin1-lmin2)


print,Fmin



pparamout(0:longueur-3,start-1)=param(0:longueur-3)
pparamout(longueur-2,start-1)=ifreq
pparamout(longueur-1,start-1)=Fmin
eerrorout(0:longueur-3,start-1)=error(0:longueur-3)
eerrorout(longueur-2,start-1)=ifreq
pparamout(0,start-1)=param(0)+Freqstart


writefits,filename+'_no',pparamout(*,0:start-1)
writefits,filename+'_no_err',eerrorout(*,0:start-1)










      endfor

end






